/*****************************************************************************\
 *
 * GUIcomponent.cpp
 *
 * GUIcomponent - Component controlling GUI 
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include "GUIcomponent.h"
#include "InputComponent.h"
#include "MyMessages.h"

#include <CEGUI/CEGUIExceptions.h>
#include <CEGUI/CEGUISchemeManager.h>
#include <CEGUI/CEGUIWindow.h>
#include <CEGUI/CEGUIWindowManager.h>
#include <dtCore/scene.h>
#include <dtUtil/log.h>
#include <osgDB/FileUtils>
#include <iostream>

GUIcomponent::GUIcomponent(dtCore::DeltaWin &win, 
						   dtCore::Keyboard &keyboard, 
						   dtCore::Mouse &mouse, 
						   const std::string &name)
	: dtGame::GMComponent(name)
	, returned(false)
	, mMainWindow(NULL)
	, mMyName(NULL)
	, mIBP(NULL)
	, mYear(NULL)
	, mIntro(NULL)
	, mNew(NULL)
	, mQuit(NULL)
	, mHelpScreen(NULL)
	, mHelpHeader(NULL)
	, mAbout(NULL)
	, mControlsH(NULL)
	, mGoalsH(NULL)
	, mControls(NULL)
	, mGoals(NULL)
	, mHelpToMenu(NULL)
	, mBackToGame(NULL)
	, mHUDmain(NULL)
	, mInvItem1(NULL)
	, mInvItem2(NULL)
	, mInvItem3(NULL)
	, mUseIcon(NULL)
	, mHint(NULL)
	, mHintNShovel(NULL)
	, mHintNKey(NULL)
	, mHintTrap(NULL)
	, mDebrief(NULL)
	, mEndToMenu(NULL)
{
	SetupGUI(win, keyboard, mouse);
}

GUIcomponent::~GUIcomponent()
{
}

//volane okamzite po pridani do GM
void GUIcomponent::OnAddedToGM()
{
	GetGameManager()->GetApplication().AddDrawable(mGUI.get());
}

//spracovanie sprav od GM - zmenit na vlastne spravy
void GUIcomponent::ProcessMessage(const dtGame::Message &message)
{
	if(message.GetMessageType() == dtGame::MessageType::INFO_MAP_LOADED)
	{
		if(game)
			ShowHUD();
	}
	else if(message.GetMessageType() == MyMessageType::STATE_PAUSED)
	{
		ShowHelpScreen();
	}
	else if(message.GetMessageType() == MyMessageType::STATE_GAME)
	{
		ShowHUD();
		game = true;
	}
	else if(message.GetMessageType() == MyMessageType::STATE_MENU)
	{
		ShowMainMenu();
		game = false;
	}
	else if(message.GetMessageType() == MyMessageType::STATE_DEBRIEF_F)
	{
		ShowDebriefing(false);
	}
	else if(message.GetMessageType() == MyMessageType::STATE_DEBRIEF_S)
	{
		ShowDebriefing(true);
	}
	else if(message.GetMessageType() == MyMessageType::HOUSE_ENTER_POSSIBLE)
	{
		ShowEnter(true);
	}
	else if(message.GetMessageType() == MyMessageType::HOUSE_ENTER_IMPOSSIBLE)
	{
		ShowEnter(false);
	}
	else if(message.GetMessageType() == MyMessageType::INTERSECTED)
	{
		ShowUse(true);
	}
	else if(message.GetMessageType() == MyMessageType::NOT_INTERSECTED)
	{
		ShowUse(false);
	}
	else if(message.GetMessageType() == MyMessageType::ITEM_COLLECTED)
	{
		if(GetGameManager()->FindActorById(message.GetAboutActorId())->GetActor()->GetName() == "Shovel")
			mInvItem2->show();
		if(GetGameManager()->FindActorById(message.GetAboutActorId())->GetActor()->GetName() == "Key")
			mInvItem3->show();
	}
	else if(message.GetMessageType() == MyMessageType::GAME_RETURN)
	{
		SetReturned(true);
	}
	else if(message.GetMessageType() == MyMessageType::VEHICLE_ENTERED)
	{
		ShowUse(false);
	}
	else if(message.GetMessageType() == MyMessageType::RESET)
	{
		mInvItem2->hide();
		mInvItem3->hide();
		ShowMainMenu();
		game = false;
	}
	else if(message.GetMessageType() == MyMessageType::SHOW_HINT)
	{
		if(GetGameManager()->FindActorById(message.GetAboutActorId())->GetActor()->GetName() == "Treasure")
			ShowHint(SHOVEL);
		if(GetGameManager()->FindActorById(message.GetAboutActorId())->GetActor()->GetName() == "Shovel")
			ShowHint(KEY);
		if(GetGameManager()->FindActorById(message.GetAboutActorId())->GetActor()->GetName() == "playerActor")
			ShowHint(TRAP);
	}
	else if(message.GetMessageType() == MyMessageType::HIDE_HINT)
	{
		HideHints();
	}
	else
	{
		//neznama sprava - nejak zareagovat
	}
}

//pomocna funkcia pre inicializaciu a zostavenie GUI
void GUIcomponent::SetupGUI(dtCore::DeltaWin &win,
							dtCore::Keyboard &keyboard,
							dtCore::Mouse &mouse)
{
	NoLogger NL;
	mGUI = new dtGUI::CEUIDrawable(&win, &keyboard, &mouse);

	try
	{
		// ------------------ !!! POZOR !!! zmenit cestu! -------------------------------
		std::string schemefile(osgDB::findDataFile("WindowsLook.scheme"));
        CEGUI::SchemeManager* sm = CEGUI::SchemeManager::getSingletonPtr();
        sm->loadScheme(schemefile);

		//inicializujeme CEGUI a vytvorime "root okno"
		CEGUI::WindowManager* wm = CEGUI::WindowManager::getSingletonPtr();
		mMainWindow = wm->createWindow("DefaultGUISheet", "root");
		CEGUI::System::getSingleton().setGUISheet(mMainWindow);
		
		//volanie vyskladania jednotlivych GUI obrazoviek
		BuildMainMenu();
		BuildHelpScreen();
		BuildHUD();
		BuildDebriefing();
	}
	catch(CEGUI::Exception &e)
	{
		dtUtil::Log::GetInstance().LogMessage(dtUtil::Log::LOG_WARNING, __FUNCTION__, "CEGUI::%s", e.getMessage().c_str());
	}
}

//skryje vsetky zlozky GUI
void GUIcomponent::HideEverything()
{
	mMainMenu->hide();
	mHelpScreen->hide();
	mHUDmain->hide();
	mDebrief->hide();
	//vsetky ostatne vetvy postupne to iste
}

//skryje hinty
void GUIcomponent::HideHints()
{
	mHintNShovel->hide();
	mHintNKey->hide();
	mHintTrap->hide();
}

//zobrazi hlavne menu
void GUIcomponent::ShowMainMenu()
{
	HideEverything();
	GetGameManager()->GetApplication().GetWindow()->ShowCursor(true);
	mMainMenu->show();
}

//zobrazi help
void GUIcomponent::ShowHelpScreen()
{
	HideEverything();
	GetGameManager()->GetApplication().GetWindow()->ShowCursor(true);
	mHelpScreen->show();
}
//zobrazi hint
void GUIcomponent::ShowHint(short hint)
{
	switch(hint)
	{
	case SHOVEL:
		mHintNShovel->show();
		break;
	case KEY:
		mHintNKey->show();
		break;
	case TRAP:
		mHintTrap->show();
		break;
	default:
		break;
	}
}

//zobrazi HUD
void GUIcomponent::ShowHUD()
{
	HideEverything();
	GetGameManager()->GetApplication().GetWindow()->ShowCursor(false);
	mHUDmain->show();
	ShowUse(false);
	mHintNShovel->hide();
	mHintNKey->hide();
	mHintTrap->hide();
	mInvItem2->hide();
	mInvItem3->hide();

	InputComponent *input = dynamic_cast<InputComponent*>(GetGameManager()->GetComponentByName("Input"));
	if(input != NULL)
	{
		if(input->GetHasShovel())//shovel
		{
			mInvItem2->show();
		}
		if(input->GetHasKey())//key
		{
			mInvItem3->show();
		}
	}//input
}

//zobrazi/skryje ikonu pouzit a hint
void GUIcomponent::ShowUse(bool show)
{
	if(show)
	{
		mUseIcon->show();
		mHint->show();
	}else{
		mUseIcon->hide();
		mHint->hide();
	}
}

//zobrazi/skryje hint o vstupe do budovy
void GUIcomponent::ShowEnter(bool show)
{
	if(show)
	{
		mEnterHint->show();
	}else{
		mEnterHint->hide();
	}
}

//zobrazenie ukoncovacej obrazovky
void GUIcomponent::ShowDebriefing(bool success)
{
	HideEverything();
	GetGameManager()->GetApplication().GetWindow()->ShowCursor(true);

	mDebrief->show();

	if(success)
	{
		mFailure->hide();
		mSuccess->show();
	}else{
		mSuccess->hide();
		mFailure->show();
	}
}

//vlastne zostavenie hlavneho menu
void GUIcomponent::BuildMainMenu()
{
	try
	{
		CEGUI::WindowManager* wm = CEGUI::WindowManager::getSingletonPtr();

		mMainMenu = wm->createWindow("WindowsLook/StaticImage", "HlavneMenu");
		mMainWindow->addChildWindow(mMainMenu);
		mMainMenu->setPosition(CEGUI::UVector2(cegui_reldim(0.0f), cegui_reldim(0.0f)));
		mMainMenu->setSize(CEGUI::UVector2(cegui_reldim(1.0f), cegui_reldim(1.0f)));
		mMainMenu->setProperty("Image", "set:MainMenuBG image:MainMenuBG");
		mMainMenu->setProperty("BackgroundEnabled", "false");
		mMainMenu->setProperty("FrameEnabled", "false");

		mMyName = wm->createWindow("WindowsLook/StaticText", "Author");
		mMyName->setText("Lukas Martanovic");
		mMyName->setSize(CEGUI::UVector2(cegui_reldim(0.25f), cegui_reldim(0.05f)));
		mMyName->setHorizontalAlignment(CEGUI::HA_RIGHT);
		mMyName->setVerticalAlignment(CEGUI::VA_BOTTOM);
		mMyName->setProperty("HorzFormatting", "HorzCentred");
		mMyName->setProperty("FrameEnabled", "false");
		mMyName->setProperty("BackgroundEnabled", "false");
		mMainMenu->addChildWindow(mMyName);

		mYear = wm->createWindow("WindowsLook/StaticText", "Year");
		mYear->setText("2011");
		mYear->setSize(CEGUI::UVector2(cegui_reldim(0.25f), cegui_reldim(0.05f)));
		mYear->setHorizontalAlignment(CEGUI::HA_LEFT);
		mYear->setVerticalAlignment(CEGUI::VA_BOTTOM);
		mYear->setProperty("HorzFormatting", "HorzCentred");
		mYear->setProperty("FrameEnabled", "false");
		mYear->setProperty("BackgroundEnabled", "false");
		mMainMenu->addChildWindow(mYear);

		mIBP = wm->createWindow("WindowsLook/StaticText", "IBP");
		mIBP->setText("IBP");
		mIBP->setSize(CEGUI::UVector2(cegui_reldim(0.25f), cegui_reldim(0.05f)));
		mIBP->setHorizontalAlignment(CEGUI::HA_CENTRE);
		mIBP->setVerticalAlignment(CEGUI::VA_BOTTOM);
		mIBP->setProperty("HorzFormatting", "HorzCentred");
		mIBP->setProperty("FrameEnabled", "false");
		mIBP->setProperty("BackgroundEnabled", "false");
		mMainMenu->addChildWindow(mIBP);

	//pridat ostatne napisy
		mIntro = static_cast<CEGUI::PushButton*>(wm->createWindow("WindowsLook/Button", "introButton"));
		mIntro->setText("About");
		mIntro->setSize(CEGUI::UVector2(cegui_reldim(0.3f), cegui_reldim(0.1f)));
		mIntro->setPosition(CEGUI::UVector2(cegui_reldim(0.15f), cegui_reldim(0.45f)));
		mIntro->setMouseCursor(NULL);
		mMainMenu->addChildWindow(mIntro);

		mNew = static_cast<CEGUI::PushButton*>(wm->createWindow("WindowsLook/Button", "newButton"));
		mNew->setText("New game");
		mNew->setSize(CEGUI::UVector2(cegui_reldim(0.3f), cegui_reldim(0.1f)));
		mNew->setPosition(CEGUI::UVector2(cegui_reldim(0.15f), cegui_reldim(0.6f)));
		mNew->setMouseCursor(NULL);
		mMainMenu->addChildWindow(mNew);

		mQuit = static_cast<CEGUI::PushButton*>(wm->createWindow("WindowsLook/Button", "quitButton"));
		mQuit->setText("Quit");
		mQuit->setSize(CEGUI::UVector2(cegui_reldim(0.3f), cegui_reldim(0.1f)));
		mQuit->setPosition(CEGUI::UVector2(cegui_reldim(0.15f), cegui_reldim(0.75f)));
		mQuit->setMouseCursor(NULL);
		mMainMenu->addChildWindow(mQuit);
	//pridat ostatne tlacidla

	//prepojime tlacidlo s funkcionalitou
		mIntro->subscribeEvent(CEGUI::PushButton::EventClicked, CEGUI::Event::Subscriber(&GUIcomponent::OnIntro, this));
		mNew->subscribeEvent(CEGUI::PushButton::EventClicked, CEGUI::Event::Subscriber(&GUIcomponent::OnNew, this));
		mQuit->subscribeEvent(CEGUI::PushButton::EventClicked, CEGUI::Event::Subscriber(&GUIcomponent::OnQuit, this));

		mMainMenu->show(); //neskor odstranit
	}
	catch(CEGUI::Exception &e)
	{
		dtUtil::Log::GetInstance().LogMessage(dtUtil::Log::LOG_WARNING, __FUNCTION__, "CEGUI::%s", e.getMessage().c_str());
	}
}

//zostavenie help obrazovky
void GUIcomponent::BuildHelpScreen()
{
	try
	{
		CEGUI::WindowManager* wm = CEGUI::WindowManager::getSingletonPtr();

		mHelpScreen = wm->createWindow("WindowsLook/StaticImage", "HelpScreen");
		mMainWindow->addChildWindow(mHelpScreen);
		mHelpScreen->setPosition(CEGUI::UVector2(cegui_reldim(0.0f), cegui_reldim(0.0f)));
		mHelpScreen->setSize(CEGUI::UVector2(cegui_reldim(1.0f), cegui_reldim(1.0f)));
		mHelpScreen->setProperty("Image", "set:WindowsLook image:Background"); 
		mHelpScreen->setProperty("BackgroundEnabled", "false");
		mHelpScreen->setProperty("FrameEnabled", "false");
		
		//napisy
		mHelpHeader = wm->createWindow("WindowsLook/StaticText", "HelpHeader");
		mHelpHeader->setText("Treasure hunter - Help");
		mHelpHeader->setSize(CEGUI::UVector2(cegui_reldim(0.3f), cegui_reldim(0.05f)));
		mHelpHeader->setHorizontalAlignment(CEGUI::HA_CENTRE);
		mHelpHeader->setVerticalAlignment(CEGUI::VA_TOP);
		mHelpHeader->setProperty("HorzFormatting", "HorzCentred");
		mHelpHeader->setProperty("FrameEnabled", "false");
		mHelpHeader->setProperty("BackgroundEnabled", "false");
		mHelpScreen->addChildWindow(mHelpHeader);

		mAbout = wm->createWindow("WindowsLook/StaticText", "AboutText");
		mAbout->setText("Players goal is to find and dig up the treasure in the exterior area of the village.\n In order to accomplish it, it is necessary to collect a shovel from between the buildings\nand a key from inside the largest building.\nPlayer can also navigate himself towards the treasure\nby using an audial detector, thus making the task easier.\nTo quicken the movement around the map it is possible to use a vehicle.");
		mAbout->setSize(CEGUI::UVector2(cegui_reldim(0.9f), cegui_reldim(0.2f)));
		mAbout->setPosition(CEGUI::UVector2(cegui_reldim(0.05f), cegui_reldim(0.075f)));
		mAbout->setProperty("HorzFormatting", "HorzCentred");
		mAbout->setProperty("FrameEnabled", "false");
		mAbout->setProperty("BackgroundEnabled", "false");
		mHelpScreen->addChildWindow(mAbout);

		mGoalsH = wm->createWindow("WindowsLook/StaticText", "Goals Header");
		mGoalsH->setText("Goals");
		mGoalsH->setSize(CEGUI::UVector2(cegui_reldim(0.2f), cegui_reldim(0.05f)));
		mGoalsH->setPosition(CEGUI::UVector2(cegui_reldim(0.0f), cegui_reldim(0.35f)));
		mGoalsH->setHorizontalAlignment(CEGUI::HA_RIGHT);
		mGoalsH->setProperty("HorzFormatting", "HorzCentred");
		mGoalsH->setProperty("FrameEnabled", "false");
		mGoalsH->setProperty("BackgroundEnabled", "false");
		mHelpScreen->addChildWindow(mGoalsH);

		mControlsH = wm->createWindow("WindowsLook/StaticText", "Controls Header");
		mControlsH->setText("Controls");
		mControlsH->setSize(CEGUI::UVector2(cegui_reldim(0.2f), cegui_reldim(0.05f)));
		mControlsH->setPosition(CEGUI::UVector2(cegui_reldim(0.0f), cegui_reldim(0.35f)));
		mControlsH->setHorizontalAlignment(CEGUI::HA_LEFT);
		mControlsH->setProperty("HorzFormatting", "HorzCentred");
		mControlsH->setProperty("FrameEnabled", "false");
		mControlsH->setProperty("BackgroundEnabled", "false");
		mHelpScreen->addChildWindow(mControlsH);

		mControls = wm->createWindow("WindowsLook/StaticText", "Controls");
		mControls->setText("Movement on foot - W,A,S,D\nMovement in vehicle - Arrow keys\nUse - F\nToggle radar (detector) outside - R\nToggle flashlight inside - L\nHelp/Main Menu - Esc");
		mControls->setSize(CEGUI::UVector2(cegui_reldim(0.4f), cegui_reldim(0.55f)));
		mControls->setPosition(CEGUI::UVector2(cegui_reldim(0.05f), cegui_reldim(0.45f)));
		mControls->setProperty("HorzFormatting", "LeftAligned");
		mControls->setProperty("VertFormatting", "TopAligned");
		mControls->setProperty("FrameEnabled", "false");
		mControls->setProperty("BackgroundEnabled", "false");
		mHelpScreen->addChildWindow(mControls);

		mGoals = wm->createWindow("WindowsLook/StaticText", "Goals");
		mGoals->setText("Find a shovel somewhere in the village -\n\
						 Enter the warehouse -\n\
						 Find a key inside -\n\
						 Use detector (radar) to find the treasure chest -\n\
						 Dig up and unlock treasure chest -\n(don't try to force it!)\n\n\
						 You can use a vehicle to move around faster -");
		mGoals->setSize(CEGUI::UVector2(cegui_reldim(0.4f), cegui_reldim(0.55f)));
		mGoals->setPosition(CEGUI::UVector2(cegui_reldim(0.55f), cegui_reldim(0.45f)));
		mGoals->setProperty("HorzFormatting", "RightAligned");
		mGoals->setProperty("VertFormatting", "TopAligned");
		mGoals->setProperty("FrameEnabled", "false");
		mGoals->setProperty("BackgroundEnabled", "false");
		mHelpScreen->addChildWindow(mGoals);

		//tlacidla
		mHelpToMenu = static_cast<CEGUI::PushButton*>(wm->createWindow("WindowsLook/Button", "menuButton"));
		mHelpToMenu->setText("Main Menu");
		mHelpToMenu->setSize(CEGUI::UVector2(cegui_reldim(0.2f), cegui_reldim(0.1f)));
		mHelpToMenu->setPosition(CEGUI::UVector2(cegui_reldim(0.3f), cegui_reldim(0.8f)));
		mHelpToMenu->setMouseCursor(NULL);
		mHelpScreen->addChildWindow(mHelpToMenu);

		mBackToGame = static_cast<CEGUI::PushButton*>(wm->createWindow("WindowsLook/Button", "gameButton"));
		mBackToGame->setText("Game");
		mBackToGame->setSize(CEGUI::UVector2(cegui_reldim(0.2f), cegui_reldim(0.1f)));
		mBackToGame->setPosition(CEGUI::UVector2(cegui_reldim(0.5f), cegui_reldim(0.8f)));
		mBackToGame->setMouseCursor(NULL);
		mHelpScreen->addChildWindow(mBackToGame);
		
		//prepojenie tlacidiel
		mHelpToMenu->subscribeEvent(CEGUI::PushButton::EventClicked, CEGUI::Event::Subscriber(&GUIcomponent::OnMainMenu, this));
		mBackToGame->subscribeEvent(CEGUI::PushButton::EventClicked, CEGUI::Event::Subscriber(&GUIcomponent::OnBackToGame, this));
	}
	catch(CEGUI::Exception &e)
	{
		dtUtil::Log::GetInstance().LogMessage(dtUtil::Log::LOG_WARNING, __FUNCTION__, "CEGUI::%s", e.getMessage().c_str());
	}

	mHelpScreen->hide();
}

//zostavenie HUD-u
void GUIcomponent::BuildHUD()
{
	try
	{
		CEGUI::WindowManager *wm = CEGUI::WindowManager::getSingletonPtr();

		mHUDmain = wm->createWindow("WindowsLook/StaticImage", "HUD");
		mMainWindow->addChildWindow(mHUDmain);
		mHUDmain->setPosition(CEGUI::UVector2(cegui_reldim(0.0f), cegui_reldim(0.0f)));
		mHUDmain->setSize(CEGUI::UVector2(cegui_reldim(1.0f), cegui_reldim(1.0f))); 
		mHUDmain->setProperty("BackgroundEnabled", "false");
		mHUDmain->setProperty("FrameEnabled", "false");

		mInvItem1 = wm->createWindow("WindowsLook/StaticImage", "InvItem1");
		mInvItem1->setSize(CEGUI::UVector2(cegui_reldim(0.1f), cegui_reldim(0.1f)));
		mInvItem1->setPosition(CEGUI::UVector2(cegui_reldim(0.85f), cegui_reldim(0.85f)));
		mInvItem1->setProperty("Image", "set:RadarIcon image:RadarIcon"); 
		mInvItem1->setProperty("BackgroundEnabled", "false");
		mInvItem1->setProperty("FrameEnabled", "false");
		mHUDmain->addChildWindow(mInvItem1);

		mInvItem2 = wm->createWindow("WindowsLook/StaticImage", "InvItem2");
		mInvItem2->setSize(CEGUI::UVector2(cegui_reldim(0.1f), cegui_reldim(0.1f)));
		mInvItem2->setPosition(CEGUI::UVector2(cegui_reldim(0.7f), cegui_reldim(0.85f)));
		mInvItem2->setProperty("Image", "set:ShovelIcon image:ShovelIcon"); 
		mInvItem2->setProperty("BackgroundEnabled", "false");
		mInvItem2->setProperty("FrameEnabled", "false");
		mHUDmain->addChildWindow(mInvItem2);

		mInvItem3 = wm->createWindow("WindowsLook/StaticImage", "InvItem3");
		mInvItem3->setSize(CEGUI::UVector2(cegui_reldim(0.1f), cegui_reldim(0.1f)));
		mInvItem3->setPosition(CEGUI::UVector2(cegui_reldim(0.55f), cegui_reldim(0.85f)));
		mInvItem3->setProperty("Image", "set:KeyIcon image:KeyIcon"); 
		mInvItem3->setProperty("BackgroundEnabled", "false");
		mInvItem3->setProperty("FrameEnabled", "false");
		mHUDmain->addChildWindow(mInvItem3);

		mUseIcon = wm->createWindow("WindowsLook/StaticImage", "UseIcon");
		mUseIcon->setSize(CEGUI::UVector2(cegui_reldim(0.1f), cegui_reldim(0.1f)));
		mUseIcon->setPosition(CEGUI::UVector2(cegui_reldim(0.45f), cegui_reldim(0.45f)));
		mUseIcon->setProperty("Image", "set:UseIcon image:UseIcon"); 
		mUseIcon->setProperty("BackgroundEnabled", "false");
		mUseIcon->setProperty("FrameEnabled", "false");
		mHUDmain->addChildWindow(mUseIcon);

		mHint = wm->createWindow("WindowsLook/StaticText", "Hint");
		mHint->setSize(CEGUI::UVector2(cegui_reldim(0.2f), cegui_reldim(0.1f)));
		mHint->setPosition(CEGUI::UVector2(cegui_reldim(0.05f), cegui_reldim(0.85f)));
		mHint->setText("Press F to interact");
		mHint->setProperty("TextColours", "tl:FFFFFFFF tr:FFFFFFFF bl:FFFFFFFF br:FFFFFFFF");
		mHint->setProperty("BackgroundEnabled", "false");
		mHint->setProperty("FrameEnabled", "false");
		mHUDmain->addChildWindow(mHint);

		mHintNShovel = wm->createWindow("WindowsLook/StaticText", "HintShovel");
		mHintNShovel->setSize(CEGUI::UVector2(cegui_reldim(0.4f), cegui_reldim(0.1f)));
		mHintNShovel->setPosition(CEGUI::UVector2(cegui_reldim(0.55f), cegui_reldim(0.75f)));
		mHintNShovel->setText("You need a shovel to dig up the treasure");
		mHintNShovel->setProperty("TextColours", "tl:FFFFFFFF tr:FFFFFFFF bl:FFFFFFFF br:FFFFFFFF");
		mHintNShovel->setProperty("HorzFormatting", "RightAligned");
		mHintNShovel->setProperty("BackgroundEnabled", "false");
		mHintNShovel->setProperty("FrameEnabled", "false");
		mHUDmain->addChildWindow(mHintNShovel);

		mHintNKey = wm->createWindow("WindowsLook/StaticText", "HintKey");
		mHintNKey->setSize(CEGUI::UVector2(cegui_reldim(0.4f), cegui_reldim(0.1f)));
		mHintNKey->setPosition(CEGUI::UVector2(cegui_reldim(0.55f), cegui_reldim(0.75f)));
		mHintNKey->setText("Now you better find a key");
		mHintNKey->setProperty("TextColours", "tl:FFFFFFFF tr:FFFFFFFF bl:FFFFFFFF br:FFFFFFFF");
		mHintNKey->setProperty("HorzFormatting", "RightAligned");
		mHintNKey->setProperty("BackgroundEnabled", "false");
		mHintNKey->setProperty("FrameEnabled", "false");
		mHUDmain->addChildWindow(mHintNKey);

		mHintTrap = wm->createWindow("WindowsLook/StaticText", "HintTrap");
		mHintTrap->setSize(CEGUI::UVector2(cegui_reldim(0.4f), cegui_reldim(0.1f)));
		mHintTrap->setPosition(CEGUI::UVector2(cegui_reldim(0.55f), cegui_reldim(0.75f)));
		mHintTrap->setText("Be careful - it might be booby trapped!");
		mHintTrap->setProperty("TextColours", "tl:FFFFFFFF tr:FFFFFFFF bl:FFFFFFFF br:FFFFFFFF");
		mHintTrap->setProperty("HorzFormatting", "RightAligned");
		mHintTrap->setProperty("BackgroundEnabled", "false");
		mHintTrap->setProperty("FrameEnabled", "false");
		mHUDmain->addChildWindow(mHintTrap);

		mEnterHint = wm->createWindow("WindowsLook/StaticText", "EnterHint");
		mEnterHint->setSize(CEGUI::UVector2(cegui_reldim(0.2f), cegui_reldim(0.1f)));
		mEnterHint->setPosition(CEGUI::UVector2(cegui_reldim(0.05f), cegui_reldim(0.85f)));
		mEnterHint->setText("Press F to enter/leave building");
		mEnterHint->setProperty("TextColours", "tl:FFFFFFFF tr:FFFFFFFF bl:FFFFFFFF br:FFFFFFFF");
		mEnterHint->setProperty("BackgroundEnabled", "false");
		mEnterHint->setProperty("FrameEnabled", "false");
		mHUDmain->addChildWindow(mEnterHint);
	}
	catch(CEGUI::Exception &e)
	{
		dtUtil::Log::GetInstance().LogMessage(dtUtil::Log::LOG_WARNING, __FUNCTION__, "CEGUI::%s", e.getMessage().c_str());
	}

	mHUDmain->hide();
}

//vystavba debriefing obrazovky
void GUIcomponent::BuildDebriefing()
{
	try
	{
		CEGUI::WindowManager *wm = CEGUI::WindowManager::getSingletonPtr();

		mDebrief = wm->createWindow("WindowsLook/StaticImage", "Debriefing");
		mMainWindow->addChildWindow(mDebrief);
		mDebrief->setPosition(CEGUI::UVector2(cegui_reldim(0.0f), cegui_reldim(0.0f)));
		mDebrief->setSize(CEGUI::UVector2(cegui_reldim(1.0f), cegui_reldim(1.0f)));
		mDebrief->setProperty("Image", "set:WindowsLook image:Background");
		mDebrief->setProperty("BackgroundEnabled", "false");
		mDebrief->setProperty("FrameEnabled", "false");

		mFailure = wm->createWindow("WindowsLook/StaticImage", "DebriefF");
		mFailure->setPosition(CEGUI::UVector2(cegui_reldim(0.0f), cegui_reldim(0.0f)));
		mFailure->setSize(CEGUI::UVector2(cegui_reldim(1.0f), cegui_reldim(1.0f)));
		mFailure->setProperty("Image", "set:DebriefF image:DebriefF");
		mFailure->setProperty("BackgroundEnabled", "false");
		mFailure->setProperty("FrameEnabled", "false");
		mDebrief->addChildWindow(mFailure);

		mSuccess = wm->createWindow("WindowsLook/StaticImage", "DebriefS");
		mSuccess->setPosition(CEGUI::UVector2(cegui_reldim(0.0f), cegui_reldim(0.0f)));
		mSuccess->setSize(CEGUI::UVector2(cegui_reldim(1.0f), cegui_reldim(1.0f)));
		mSuccess->setProperty("Image", "set:DebriefS image:DebriefS");
		mSuccess->setProperty("BackgroundEnabled", "false");
		mSuccess->setProperty("FrameEnabled", "false");
		mDebrief->addChildWindow(mSuccess);

		mEndToMenu = static_cast<CEGUI::PushButton*>(wm->createWindow("WindowsLook/Button", "EndMenuButton"));
		mEndToMenu->setText("Main Menu");
		mEndToMenu->setSize(CEGUI::UVector2(cegui_reldim(0.2f), cegui_reldim(0.1f)));
		mEndToMenu->setPosition(CEGUI::UVector2(cegui_reldim(0.1f), cegui_reldim(0.8f)));
		mEndToMenu->setMouseCursor(NULL);
		mDebrief->addChildWindow(mEndToMenu);

		mEndToMenu->subscribeEvent(CEGUI::PushButton::EventClicked, CEGUI::Event::Subscriber(&GUIcomponent::OnMainMenu, this));
	}
	catch(CEGUI::Exception &e)
	{
		dtUtil::Log::GetInstance().LogMessage(dtUtil::Log::LOG_WARNING, __FUNCTION__, "CEGUI::%s", e.getMessage().c_str());
	}

	mDebrief->hide();
}

//obsluha tlacidla INTRO
bool GUIcomponent::OnIntro(const CEGUI::EventArgs &e)
{
	dtCore::RefPtr<dtGame::Message> msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::STATE_PAUSED);
	GetGameManager()->SendMessage(*msg);
	return true;
}

//obsluha tlacidla NEW
bool GUIcomponent::OnNew(const CEGUI::EventArgs &e)
{
	//zmena stavu na game -> spustenie hry
	dtCore::RefPtr<dtGame::Message> msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::STATE_GAME);
	GetGameManager()->SendMessage(*msg);
	return true;
}

//obsluha tlacidla QUIT
bool GUIcomponent::OnQuit(const CEGUI::EventArgs &e)
{
	GetGameManager()->GetApplication().Quit();
	return true;
}

//obsluha tlacidla MAIN MENU
bool GUIcomponent::OnMainMenu(const CEGUI::EventArgs &e)
{
	//zmena stavu na main menu -> prerusenie hry, resp. zobrazenie hlavneho menu v zavislosti na aktualnom stave
	dtCore::RefPtr<dtGame::Message> msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::RESET);
	GetGameManager()->SendMessage(*msg);
	return true;
}

//obsluha tlacidla RETURN TO GAME
bool GUIcomponent::OnBackToGame(const CEGUI::EventArgs &e)
{
	//zmena stavu HELP->GAME
	dtCore::RefPtr<dtGame::Message> msg1 = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::GAME_RETURN);
	GetGameManager()->SendMessage(*msg1);
	dtCore::RefPtr<dtGame::Message> msg2 = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::STATE_GAME);
	GetGameManager()->SendMessage(*msg2);
	return true;
}

//nastavi priznak "navrat z pause"
void GUIcomponent::SetReturned(bool set)
{
	returned = set;
}

//zisti, ci sme po navrate z help screen
bool GUIcomponent::IsReturned()
{
	return returned;
}