/*****************************************************************************\
 *
 * GUIcomponent.h
 *
 * GUIcomponent - Component controlling GUI 
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#ifndef GUI_COMPONENT_H
#define GUI_COMPONENT_H

#include <CEGUI.h>
#include <dtCore/deltawin.h>
#include <dtCore/keyboard.h>
#include <dtCore/mouse.h>
#include <dtGame/gmcomponent.h>
#include <dtGame/gameapplication.h>
#include <dtGame/message.h>
#include <dtGUI/ceuidrawable.h>
#include "export.h"

#define SHOVEL 0
#define KEY 1
#define TRAP 2

class NoLogger : public CEGUI::Logger
{
	void logEvent (const CEGUI::String&, CEGUI::LoggingLevel){}
	void setLogFilename(const CEGUI::String&, bool){}
};

class IBP_EXPORT GUIcomponent : public dtGame::GMComponent
{
public:
	GUIcomponent(dtCore::DeltaWin &win, dtCore::Keyboard &keyboard, dtCore::Mouse &mouse, const std::string &name);

	virtual void OnAddedToGM();
	virtual void ProcessMessage(const dtGame::Message &message);

protected:
	~GUIcomponent();
	void HideEverything();
	void HideHints();

	void ShowMainMenu();				//zobrazi hlavne menu
	void ShowEnter(bool show);			//zobrazi/skryje napovedu o vstupe do budovy
	void ShowHelpScreen();				//zobrazi help
	void ShowHint(short hint);			//zobrazi hint
	void ShowHUD();						//zobrazi HUD
	void ShowUse(bool show);			//zobrazi/skryje ikonu pouzit a hint
	void ShowDebriefing(bool success);	//ukoncovacia obrazovka

private:
	void SetupGUI(dtCore::DeltaWin &win, dtCore::Keyboard &keyboard, dtCore::Mouse &mouse); //pomocna funkcia
	
	void BuildMainMenu();	//vlastne zostavenie hlavneho menu
	void BuildHelpScreen();	//zostavenie help obrazovky
	void BuildHUD();		//HUD v hre
	void BuildDebriefing();	//ukoncovacia obrazovka

	bool OnIntro(const CEGUI::EventArgs &e);	//reakcia na tlacidlo INTRO
	bool OnNew(const CEGUI::EventArgs &e);	//reakcia na tlacidlo NEW
	bool OnQuit(const CEGUI::EventArgs &e);	//reakcia na stlacenie tlacidla QUIT v hlavnom menu
	bool OnMainMenu(const CEGUI::EventArgs &e);	//reakcia na tlacidlo pre navrat do hlavneho menu
	bool OnBackToGame(const CEGUI::EventArgs &e); //reakcia na tlacidlo spat do hry v help

	void SetReturned(bool set);
	bool IsReturned();

	bool returned;
	bool game;

	CEGUI::Window *mMainWindow; //root
	CEGUI::Window *mMainMenu; //vetva main menu
	CEGUI::Window *mMyName, *mIBP, *mYear;	//napisy v hlavnom menu
	CEGUI::PushButton *mIntro, *mNew, *mQuit;			//tlacidla v hlavnom menu

	CEGUI::Window *mHelpScreen;	//vetva help
	CEGUI::Window *mAbout; //strucny popis o co ide
	CEGUI::Window *mHelpHeader, *mControlsH, *mGoalsH, *mControls, *mGoals;		//napisy v help - pribudnu dalsie
	CEGUI::PushButton *mHelpToMenu, *mBackToGame;		//tlacidlo navrat do hlavneho menu

	CEGUI::Window *mHUDmain;	//vetva HUD
	CEGUI::Window *mInvItem1, *mInvItem2, *mInvItem3;	//polozky v inventari - zatial testovacie
	CEGUI::Window *mUseIcon, *mHint, *mEnterHint;
	CEGUI::Window *mHintNShovel, *mHintNKey, *mHintTrap;

	CEGUI::Window *mDebrief;	//vetva debriefing
	CEGUI::Window *mEndToMenu, *mFailure, *mSuccess;	//napis o splneni/nesplneni a tlacidlo pre navrat do menu

	dtCore::RefPtr<dtGUI::CEUIDrawable> mGUI;
};

#endif