/*****************************************************************************\
 *
 * IBPGameEntryPoint.cpp
 *
 * IBPGameEntryPoint - Class representing entry and initialisation point for 
 *   				   application
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include "GUIcomponent.h"
#include "InputComponent.h"
#include "IBPGameEntryPoint.h"
#include "MyMessages.h"

#include <dtAudio/audiomanager.h>
#include <dtCore/globals.h>
#include <dtCore/refptr.h>
#include <dtCore/transform.h>	//odstranit akonahle sa kamera nastavi inde
#include <dtDAL/project.h>
#include <dtGame/defaultmessageprocessor.h>
#include <dtGame/message.h>
#include <dtUtil/log.h>
#include <iostream>

extern "C" IBP_EXPORT dtGame::GameEntryPoint* CreateGameEntryPoint()
{
   return new IBPGameEntryPoint;
}

extern "C" IBP_EXPORT void DestroyGameEntryPoint(dtGame::GameEntryPoint* entryPoint)
{
   delete entryPoint;
}

IBPGameEntryPoint::IBPGameEntryPoint()
{}

IBPGameEntryPoint::~IBPGameEntryPoint()
{}

void IBPGameEntryPoint::Initialize(dtGame::GameApplication &app, int argc, char **argv)
{
	//nastavenie kontextu a ciest
	dtDAL::Project::GetInstance().SetContext(".");
	dtCore::SetDataFilePathList( "..;" + dtDAL::Project::GetInstance().GetContext() + ";" +
		dtDAL::Project::GetInstance().GetContext() + "/GUI;" + 
		dtDAL::Project::GetInstance().GetContext() + "/maps;" + 
		dtDAL::Project::GetInstance().GetContext() + "/Particles;" +
		dtDAL::Project::GetInstance().GetContext() + "/Shaders;" +
		dtDAL::Project::GetInstance().GetContext() + "/Sounds;" + 
		dtDAL::Project::GetInstance().GetContext() + "/StaticMeshes;" + 
		dtDAL::Project::GetInstance().GetContext() + "/Textures");

	//nastavenie parametrov okna aplikacie
	app.GetWindow()->SetWindowTitle("IBP - Delta3D");
	app.GetWindow()->SetFullScreenMode(false);
}

void IBPGameEntryPoint::OnStartup(dtGame::GameApplication &app)
{
	//zavedenie AudioManagera
	dtAudio::AudioManager::Instantiate();
	app.GetCamera()->AddChild(dtAudio::AudioManager::GetListener());	//pocuvame z pozicie kamery

	//GM
	dtGame::GameManager& gameManager = *app.GetGameManager();

	//registracia sprav
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::STATE_MENU);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::STATE_GAME);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::STATE_PAUSED);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::STATE_FLYBY);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::STATE_DEBRIEF_S);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::STATE_DEBRIEF_F);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::INTERSECTED);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::NOT_INTERSECTED);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::ITEM_COLLECTED);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::TOGGLE_RADAR);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::GAME_RETURN);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::VEHICLE_ENTERED);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::VEHICLE_LEFT);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::RESET);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::VEHICLE_MOVES);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::VEHICLE_STOPPED);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::SHOW_HINT);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::HIDE_HINT);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::HOUSE_ENTER_POSSIBLE);
	gameManager.GetMessageFactory().RegisterMessageType<dtGame::Message>(MyMessageType::HOUSE_ENTER_IMPOSSIBLE);

	//vytvorenie komponentov
	dtCore::RefPtr<GUIcomponent> GUIcomp = new GUIcomponent(*app.GetWindow(), *app.GetKeyboard(), *app.GetMouse(), "GUI");
	dtCore::RefPtr<InputComponent> InpComp = new InputComponent("Input");
	dtCore::RefPtr<dtGame::DefaultMessageProcessor> dmp = new dtGame::DefaultMessageProcessor("DefaultMessageProcessor");

	//pripojenie komponentov
	gameManager.AddComponent(*dmp, dtGame::GameManager::ComponentPriority::HIGHEST);
	gameManager.AddComponent(*InpComp, dtGame::GameManager::ComponentPriority::NORMAL);
	gameManager.AddComponent(*GUIcomp, dtGame::GameManager::ComponentPriority::NORMAL);

	//nastavenie kontextu a nacitanie mapy - zavisle na stave, pravdepodobne sucast niektoreho komponentu vo finale
	gameManager.ChangeMap("Map");

	//kompozicia a odoslanie spravy o zmene stavu na MENU
	dtCore::RefPtr<dtGame::Message> msg = gameManager.GetMessageFactory().CreateMessage(MyMessageType::STATE_MENU);
	gameManager.SendMessage(*msg);
}