/*****************************************************************************\
 *
 * IBPgameActorsRegistry.cpp
 *
 * IBPgameActorsRegistry - Class for registering Actors in Game Manager
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include "IBPgameActorsRegistry.h"
#include "playerActor.h"
#include "interactiveActor.h"
#include "collectibleActor.h"
#include "enterTriggerActor.h"
#include "radarActor.h"
#include "treasureActor.h"
#include "vehicleActor.h"

extern "C" IBP_EXPORT dtDAL::ActorPluginRegistry* CreatePluginRegistry()
{
	return new IBPgameActorsRegistry();
}

extern "C" IBP_EXPORT void DestroyPluginRegistry(dtDAL::ActorPluginRegistry *registry)
{
	delete registry;
}

IBPgameActorsRegistry::IBPgameActorsRegistry() : dtDAL::ActorPluginRegistry("IBPlibrary")
{
	SetDescription("IBP - Delta3D project library");
}

//registracia actorov
void IBPgameActorsRegistry::RegisterActorTypes()
{
	PlayerActorType = new dtDAL::ActorType("PlayerActor", "IBPactors", "Actor representing player entity");
	InteractiveActorType = new dtDAL::ActorType("InteractiveActor", "IBPactors", "Actor representing basic interactive object");
	CollectibleActorType = new dtDAL::ActorType("CollectibleActor", "IBPactors", "Actor representing object that can be picked up and added to players inventory");
	EnterTriggerActorType = new dtDAL::ActorType("EnterTriggerActor", "IBPactors", "Trigger enabling/disabling map change");
	RadarActorType = new dtDAL::ActorType("RadarActor", "IBPactors", "Actor that audially signalizes distance between two points");
	TreasureActorType = new dtDAL::ActorType("TreasureActor", "IBPactors", "Actor that provides user interaction");
	VehicleActorType = new dtDAL::ActorType("VehicleActor", "IBPactors", "Actor representing usable vehicle");

	mActorFactory->RegisterType<PlayerActorProxy>(PlayerActorType.get());
	mActorFactory->RegisterType<InteractiveActorProxy>(InteractiveActorType.get());
	mActorFactory->RegisterType<CollectibleActorProxy>(CollectibleActorType.get());
	mActorFactory->RegisterType<EnterTriggerActorProxy>(EnterTriggerActorType.get());
	mActorFactory->RegisterType<RadarActorProxy>(RadarActorType.get());
	mActorFactory->RegisterType<TreasureActorProxy>(TreasureActorType.get());
	mActorFactory->RegisterType<VehicleActorProxy>(VehicleActorType.get());
}