/*****************************************************************************\
 *
 * InputComponent.cpp
 *
 * InputComponent - Component controlling user input and game rules 
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include <dtCore/base.h>
#include <dtCore/scene.h>
#include <dtCore/system.h>
#include <dtCore/transformable.h>
#include <dtGame/gameapplication.h>
#include <iostream>
#include "InputComponent.h"
#include "collectibleActor.h"
#include "treasureActor.h"
#include "vehicleActor.h"

#include <osg/Texture2D>
#include <osgDB/ReadFile>

InputComponent::InputComponent(const std::string &name)
	: dtGame::BaseInputComponent(name)
	, player(NULL)
	, radar(NULL)
	, trigger(NULL)
	, goal(NULL)
	, light(NULL)
	, fmm(NULL)
	, wmm(NULL)
	, intersectedObject(NULL)
	, wantChange(false)
	, houseEnter(false)
	, changeMap1(false)
	, changeMap2(false)
	, treasure_used(false)
	, inGame(false)
	, reset(false)
	, inv_key(false)
	, inv_shovel(false)
{
}

InputComponent::~InputComponent()
{
}

//po upovedomeni GM o existencii komponentu
void InputComponent::OnAddedToGM()
{
	dtGame::BaseInputComponent::OnAddedToGM();
}

//spracovanie sprav
void InputComponent::ProcessMessage(const dtGame::Message &message)
{
	if(message.GetMessageType() == dtGame::MessageType::INFO_TIMER_ELAPSED)
	{
		if(!wantChange){
			if(player){
				if(message.GetAboutActorId() != player->GetUniqueId())
				{
					if(!inv_key)
						SendInteractionMessage(MyMessageType::STATE_DEBRIEF_F);
				}
			}//player
		}//wantChange
		else wantChange = false;
	}
	else if(message.GetMessageType() == dtGame::MessageType::INFO_MAP_LOADED)
	{
		if(GetGameManager()->GetCurrentMap() == "Map")
		{
			ChangeMap();
			changeMap1 = true;
		}
		if(GetGameManager()->GetCurrentMap() == "Map2")
		{
			ChangeMap();
			changeMap2 = true;
		}
	}
	else if(message.GetMessageType() == MyMessageType::HOUSE_ENTER_POSSIBLE)
	{
		houseEnter = true;
	}
	else if(message.GetMessageType() == MyMessageType::HOUSE_ENTER_IMPOSSIBLE)
	{
		houseEnter = false;
	}
	else if(message.GetMessageType() == MyMessageType::STATE_GAME)
	{
		inGame = true;
		if(fmm)
			fmm->SetEnabled(true);
	}//STATE_GAME
	else if(message.GetMessageType() == MyMessageType::STATE_MENU)
	{
		inGame = false;
		if(fmm)
			fmm->SetEnabled(false);	//vypneme motionmodel (uvolnime mys)
	}//STATE_MENU
	else if(message.GetMessageType() == MyMessageType::STATE_PAUSED)
	{
		inGame = false;
		if(fmm)
			fmm->SetEnabled(false);	//vypneme motionmodel (uvolnime mys)
	}//STATE_PAUSED
	else if(message.GetMessageType() == MyMessageType::STATE_DEBRIEF_F)
	{
		inGame = false;
		if(fmm)
			fmm->SetEnabled(false);	//vypneme motionmodel (uvolnime mys)
	}//STATE_DEBRIEF_F
	else if(message.GetMessageType() == MyMessageType::STATE_DEBRIEF_S)
	{
		inGame = false;
		if(fmm)
			fmm->SetEnabled(false);	//vypneme motionmodel (uvolnime mys)
	}//STATE_DEBRIEF_S
	else if(message.GetMessageType() == MyMessageType::INTERSECTED)
	{
		//poznacime si objekt na ktory sa prave pozerame
		intersectedObject = dynamic_cast<InteractiveActor*>(GetGameManager()->FindActorById(message.GetAboutActorId())->GetActor());
	}
	else if(message.GetMessageType() == MyMessageType::NOT_INTERSECTED)
	{
		intersectedObject = NULL;
	}
	else if(message.GetMessageType() == MyMessageType::RESET)
	{
		if(player && player->IsInVehicle()){
			SendInteractionMessage(MyMessageType::VEHICLE_LEFT);
			SendInteractionMessage(MyMessageType::VEHICLE_STOPPED);
			VehicleAction(false);
		}
		Initialize();
		reset = true;
		GetGameManager()->ChangeMap("Map");
	}
}

//obsluha stlacenia klaves
bool InputComponent::HandleKeyPressed(const dtCore::Keyboard *keyboard, int key)
{
	bool handled = true;
	dtCore::RefPtr<dtGame::Message> msg;

	if(inGame){
	switch(key)
	{
	//prerusenie hry
	case osgGA::GUIEventAdapter::KEY_Escape:
		msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::STATE_PAUSED);
		GetGameManager()->SendMessage(*msg);
		break;
	//pohyb vozidla
	case osgGA::GUIEventAdapter::KEY_Up:
		if(player)
		{
			if(player->IsInVehicle())
			{
				msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::VEHICLE_MOVES);
				GetGameManager()->SendMessage(*msg);
			}
		}
		break;
	case osgGA::GUIEventAdapter::KEY_Down:
		if(player)
		{
			if(player->IsInVehicle())
			{
				msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::VEHICLE_MOVES);
				GetGameManager()->SendMessage(*msg);
			}
		}
		break;
	case osgGA::GUIEventAdapter::KEY_Left:
		if(player)
		{
			if(player->IsInVehicle())
			{
				msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::VEHICLE_MOVES);
				GetGameManager()->SendMessage(*msg);
			}
		}
		break;
	case osgGA::GUIEventAdapter::KEY_Right:
		if(player)
		{
			if(player->IsInVehicle())
			{
				msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::VEHICLE_MOVES);
				GetGameManager()->SendMessage(*msg);
			}
		}
		break;
	//interakcia s okolim hraca
	case 'f': //podrobnejsie delenie podla konkretneho typu objektu (prekastit podla kontextu)
		if(!player->IsInVehicle())
		{
			if(houseEnter)
			{
				wantChange = true;
				if(GetGameManager()->GetCurrentMap() == "Map2")
				{
					GetGameManager()->ChangeMap("Map");
				}
				if(GetGameManager()->GetCurrentMap() == "Map")
				{
					GetGameManager()->ChangeMap("Map2");
				}
				houseEnter = false;
			}
			else if(intersectedObject)
			{
				CollectibleActor *ca = dynamic_cast<CollectibleActor*>(intersectedObject);
				VehicleActor *va = dynamic_cast<VehicleActor*>(intersectedObject);
				TreasureActor *ta = dynamic_cast<TreasureActor*>(intersectedObject);
				if(ca)
				{
					SendInteractionMessage(MyMessageType::ITEM_COLLECTED);
					if(ca->GetName() == "Key")
						inv_key = true;
					if(ca->GetName() == "Shovel")
					{
						inv_shovel = true;
						//zobrazime hint
						lastInteraction = ca->GetUniqueId();
						SendInteractionMessage(MyMessageType::SHOW_HINT);
						GetGameManager()->SetTimer("hint", &player->GetGameActorProxy(), 3.f, false, false);
					}//shovel
				}//ca
				else if(va)
				{
					lastInteraction = va->GetUniqueId();	//ulozime si identifikator vozidla pre potrebu vystupenia
					SendInteractionMessage(MyMessageType::VEHICLE_ENTERED);
					VehicleAction(true);
				}//va
				else if(ta)
				{
					if(ta->GetName() == "Treasure")	//poklad
					{
						if(!ta->IsUsed())	//vykopeme poklad zo zeme
							if(player->GetHasShovel())
							{
								ta->Use(false);			//"vykopeme" poklad
								treasure_used = true;
								//zobrazime hint
								lastInteraction = player->GetUniqueId();
								GetGameManager()->SetTimer("hint", &player->GetGameActorProxy(), 3.f, false, false);
								SendInteractionMessage(MyMessageType::SHOW_HINT);
							}
							else{ //zobrazime hint
								lastInteraction = ta->GetUniqueId();
								GetGameManager()->SetTimer("hint", &player->GetGameActorProxy(), 3.f, false, false);
								SendInteractionMessage(MyMessageType::SHOW_HINT);
							}
						else{				//pokusime sa odomknut
							if(player->GetHasKey())
							{
								SendInteractionMessage(MyMessageType::STATE_DEBRIEF_S);
							}else{
								fmm->SetEnabled(false);	//znehybnime hraca
								ta->Explode();			//explozia
								GetGameManager()->SetTimer("explosion", NULL, 1.2f, false, false);	//sprava o prehre
							}
						}//used
					}//ta
				}//ua
				else std::cout << "Unknown actor type" << std::endl;
			}//intersectedObject
		}//inVehicle
		else{
			SendInteractionMessage(MyMessageType::VEHICLE_LEFT);
			SendInteractionMessage(MyMessageType::VEHICLE_STOPPED);
			VehicleAction(false);
		}
		break;
	case 'l':
		if(GetGameManager()->GetCurrentMap() == "Map2")
		{
			if(light)//svetlo uz je inicializovane
			{
				light->SetEnabled(!light->GetEnabled());
			}
		}
		break;
	//zapnutie/vypnutie radaru
	case 'r':		//radar
		msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::TOGGLE_RADAR);
		GetGameManager()->SendMessage(*msg);
		break;
	default:
		handled = false;
		break;
	}//switch
	}//inGame

	return handled;
}

//uvolnenie klavesy
bool InputComponent::HandleKeyReleased(const dtCore::Keyboard *keyboard, int key)
{
	bool handled = true;
	dtCore::RefPtr<dtGame::Message> msg;

	switch(key)
	{
	case osgGA::GUIEventAdapter::KEY_Up:
		if(player)
		{
			if(player->IsInVehicle())
			{
				msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::VEHICLE_STOPPED);
				GetGameManager()->SendMessage(*msg);
			}
		}
		break;
	case osgGA::GUIEventAdapter::KEY_Down:
		if(player)
		{
			if(player->IsInVehicle())
			{
				msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::VEHICLE_STOPPED);
				GetGameManager()->SendMessage(*msg);
			}
		}
		break;
	case osgGA::GUIEventAdapter::KEY_Left:
		if(player)
		{
			if(player->IsInVehicle())
			{
				msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::VEHICLE_STOPPED);
				GetGameManager()->SendMessage(*msg);
			}
		}
		break;
	case osgGA::GUIEventAdapter::KEY_Right:
		if(player)
		{
			if(player->IsInVehicle())
			{
				msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::VEHICLE_STOPPED);
				GetGameManager()->SendMessage(*msg);
			}
		}
		break;
	default:
		handled = false;
		break;
	}
	return handled;
}

//vrati uniqueid predmetu, s ktorym sme naposledy interagovali
dtCore::UniqueId InputComponent::GetLastInteraction()
{
	return lastInteraction;
}

//inventar
bool InputComponent::GetHasKey()
{
	return inv_key;
}

bool InputComponent::GetHasShovel()
{
	return inv_shovel;
}

//zmena mapy
void InputComponent::ChangeMap()
{
	std::vector<dtGame::GameActorProxy*> proxies;
	GetGameManager()->GetAllGameActors(proxies);
	
	GetGameManager()->GetApplication().GetCamera()->Emancipate();
	player = NULL;
	radar = NULL;
	goal = NULL;
	trigger = NULL;
	intersectedObject = NULL;
	lastInteraction = "";

	if(!player)
		SetupPlayer();
	if(!fmm)
		SetupMotionModel();
	fmm->SetTarget(player);
	fmm->SetScene(&GetGameManager()->GetScene());
	fmm->SetEnabled(true);

	if(GetGameManager()->GetCurrentMap() == "Map")
	{
		GetGameManager()->GetApplication().GetScene()->UseSceneLight(true);
		SetupVehicles();
		SetupTreasure();
		if(!trigger)
			SetupTrigger();
		if(reset || (changeMap1==false))	//New game alebo reset
		{
			Initialize();
			dtCore::RefPtr<dtGame::Message> msg = GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::STATE_MENU);
			GetGameManager()->SendMessage(*msg);
			reset = false;
		}else{								//navrat z budovy
			for(unsigned int i = 0; i < proxies.size(); i++)
			{
				if(proxies[i]->GetName() == "Shovel")
				{
					if(inv_shovel)
					{
						CollectibleActor *ca = dynamic_cast<CollectibleActor*>(proxies[i]->GetActor());
						if(ca)
						{
							ca->SetCollected(true);
						}
					}//inv
				}//shovel
			}//for
		}//reset
	}//Map
	if(GetGameManager()->GetCurrentMap() == "Map2")
	{
		SetupLight();
		if(!trigger)
			SetupTrigger();
		for(unsigned int i = 0; i < proxies.size(); i++)
		{
			if(proxies[i]->GetName() == "Key")
			{
				if(inv_key)
				{
					CollectibleActor *ca = dynamic_cast<CollectibleActor*>(proxies[i]->GetActor());
					if(ca)
					{
						ca->SetCollected(true);
					}
				}//inv
			}//key
		}//for
		changeMap2 = true;
	}//map2
}

//inicializacia komponentu
void InputComponent::Initialize()
{	
	inv_key = false;
	inv_shovel = false;
	inGame = false;
	treasure_used = false;
}

//konstrukcia a odoslanie spravy o interakcii s okolim
void InputComponent::SendInteractionMessage(const dtGame::MessageType & type)
{
	//nastavime id objektu ktoreho sa interakcia dotyka
	dtCore::UniqueId id;
	if((type == MyMessageType::ITEM_COLLECTED || type == MyMessageType::VEHICLE_ENTERED) && (intersectedObject != NULL))
	{
		id = intersectedObject->GetUniqueId();
	}
	else if(type == MyMessageType::VEHICLE_LEFT || type == MyMessageType::SHOW_HINT)
	{
		id = lastInteraction;
	}
	else id = "";

	//kompozicia a odoslanie spravy o interakcii
	dtCore::RefPtr<dtGame::Message> msg = GetGameManager()->GetMessageFactory().CreateMessage(type);
	msg->SetAboutActorId(id);
	GetGameManager()->SendMessage(*msg);
}

//nastavenie "baterky" v druhej mape
void InputComponent::SetupLight()
{
	light = new dtCore::SpotLight(1);
	if(player && light)
	{
		light->SetSpotCutoff(15.0);
		light->SetEnabled(true);
		
		player->AddChild(light);
		light->SetTransform(dtCore::Transform(0.f, -0.5f, 0.f, 0.f, 0.f,0.f), dtCore::Transformable::REL_CS);
	}
	GetGameManager()->GetApplication().GetScene()->UseSceneLight(false);
}

//nastavenie a inicializacia motionmodelu
void InputComponent::SetupMotionModel()
{
	if(!fmm)
	{
		fmm = new dtCore::FPSMotionModel(GetGameManager()->GetApplication().GetKeyboard(),
										 GetGameManager()->GetApplication().GetMouse(), 10.f, 1.5f, 7.5f, 1.5f, 0.6);
		if(player)
			fmm->SetTarget(player);
		fmm->SetScene(&GetGameManager()->GetScene());
	}
	if(!wmm)
	{
		wmm = new dtCore::WalkMotionModel(GetGameManager()->GetApplication().GetKeyboard(),
										  GetGameManager()->GetApplication().GetMouse());
		wmm->SetTarget(NULL);
		wmm->SetMaximumWalkSpeed(20.f);
		wmm->SetMaximumSidestepSpeed(0.f);
		wmm->SetScene(&GetGameManager()->GetScene());
		wmm->SetHeightAboveTerrain(0.45);
	}
}

//nastavenie a inicializacia hraca
void InputComponent::SetupPlayer()
{
	std::vector<dtGame::GameActorProxy*> proxies;

	GetGameManager()->GetAllGameActors(proxies);

	for(unsigned int i = 0; i < proxies.size(); i++)
	{
		if(proxies[i]->GetName() == "playerActor")
		{
			player = dynamic_cast<PlayerActor*>(proxies[i]->GetActor());
		}//playerActor found
		if(proxies[i]->GetName() == "radarActor")
		{
			radar = dynamic_cast<RadarActor*>(proxies[i]->GetActor());
		}
		//provizorne zvoleny objekt na otestovanie radaru
		if(proxies[i]->GetName() == "Treasure")
		{
			goal = dynamic_cast<TreasureActor*>(proxies[i]->GetActor());
		}
	}//for

	if(player)
	{
		if(changeMap2 && (GetGameManager()->GetCurrentMap() == "Map") && !reset)
			player->SetTransform(dtCore::Transform(0.7f, 16.5f, 4.f, 0.f, 0.f, 0.f));
		player->Init();
		if(inv_key){
			player->SetHasKey(true);
		}
		if(inv_shovel){
			player->SetHasShovel(true);
		}
		//inicializacia radaru
		if(radar)
		{
			radar->Init();
			dtCore::Transform xform;
			osg::Vec3 goalPos;
			player->GetTransform(xform);
			player->AddChild(radar);
			radar->SetTransform(xform);
			if(goal)
			{
				goal->GetTransform(xform);
				xform.GetTranslation(goalPos);
				radar->SetGoal(goalPos);
			}
		}
	}//player cast
}

//inicializacia pokladu
void InputComponent::SetupTreasure()
{
	TreasureActor *ta;
	std::vector<dtGame::GameActorProxy*> proxies;

	GetGameManager()->GetAllGameActors(proxies);

	for(unsigned int i = 0; i < proxies.size(); i++)
	{
		if(proxies[i]->GetName().find("Treasure") != std::string::npos)
		{
			ta = dynamic_cast<TreasureActor*>(proxies[i]->GetActor());
			if(ta)
			{
				ta->Init();
				if(treasure_used)
					ta->Use(false);
			}//cast
		}//treasure
	}//for
}

//inicializacia triggeru
void InputComponent::SetupTrigger()
{
	std::vector<dtGame::GameActorProxy*> proxies;

	GetGameManager()->GetAllGameActors(proxies);

	//prejdeme vsetky vozidla
	for(unsigned int i = 0; i < proxies.size(); i++)
	{
		if(proxies[i]->GetName().find("HouseTrigger") != std::string::npos)
		{
			trigger = dynamic_cast<EnterTriggerActor*>(proxies[i]->GetActor());
			if(trigger && player)
			{
				trigger->Init(player->GetUniqueId());		//inicializujeme
			}//cast
		}//trigger
	}//for

}

//inicializacia vozidiel v scene
void InputComponent::SetupVehicles()
{
	VehicleActor *veh;
	std::vector<dtGame::GameActorProxy*> proxies;

	GetGameManager()->GetAllGameActors(proxies);

	//prejdeme vsetky vozidla
	for(unsigned int i = 0; i < proxies.size(); i++)
	{
		if(proxies[i]->GetName().find("Vehicle") != std::string::npos)
		{
			veh = dynamic_cast<VehicleActor*>(proxies[i]->GetActor());
			if(veh)
			{
				if(changeMap2 && !reset)
					veh->SetTransform(VehiclePosition);
				veh->Init();		//inicializujeme
				veh->GetTransform(VehiclePosition);
			}//cast
		}//vehicle
	}//for
}

//interakcia s vozidlom
void InputComponent::VehicleAction(bool enter)
{
	VehicleActor *va; 
	va = dynamic_cast<VehicleActor*>(intersectedObject);
	if(player){
	if(enter)	//nastup do vozidla
	{
		if(player->GetParent() != va)
		{
			va->AddChild(player);
		}
		player->SetTransform(dtCore::Transform(0.f, -10.f, 4.f, 0.f, -10.f, 0.f), dtCore::Transformable::REL_CS);
		if(fmm && wmm){
			fmm->SetTarget(NULL);	//nehybeme hracom
			wmm->SetTarget(va);		//hybeme vozidlom
		}
	}else{		//vystup z vozidla
		player->SetTransform(dtCore::Transform(0.f, -5.f, 1.5f, 0.f, 0.f, 0.f), dtCore::Transformable::REL_CS);
		
		dtCore::Transform xform;

		player->GetTransform(xform);
		player->Emancipate();
		GetGameManager()->GetApplication().GetScene()->AddDrawable(player);
		player->SetTransform(xform);

		va->GetTransform(VehiclePosition);	//ulozime aktualnu poziciu vozidla
		if(fmm && wmm){
			fmm->SetTarget(player);	//hybeme hracom
			wmm->SetTarget(NULL);	//nehybeme vozidlom
		}
	}//action
	}//player
}