/*****************************************************************************\
 *
 * InputComponent.h
 *
 * InputComponent - Component controlling user input and game rules 
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#ifndef INPUT_COMPONENT_H
#define INPUT_COMPONENT_H

#include <dtCore/keyboard.h>
#include <dtCore/FPSmotionmodel.h>
#include <dtCore/spotlight.h>
#include <dtCore/walkmotionmodel.h>
#include <dtGame/baseinputcomponent.h>
#include <dtGame/message.h>
#include "export.h"
#include "EnterTriggerActor.h"
#include "MyMessages.h"
#include "playerActor.h"
#include "radarActor.h"
#include "interactiveActor.h"

class IBP_EXPORT InputComponent : public dtGame::BaseInputComponent
{
public:
	InputComponent(const std::string &name);

	virtual void OnAddedToGM();
	virtual void ProcessMessage(const dtGame::Message& message);

	virtual bool HandleKeyPressed(const dtCore::Keyboard* keyboard, int key);	//stlacenie klavesy
	virtual bool HandleKeyReleased(const dtCore::Keyboard* keyboard, int key);  //uvolnenie klavesy

	dtCore::UniqueId GetLastInteraction();		//vrati ID predmetu, s ktorym sme naposledy interagovali
	bool GetHasKey();
	bool GetHasShovel();

protected:
	~InputComponent();

private:
	void Initialize();
	void ChangeMap();	//zmena mapy
	void SendInteractionMessage(const dtGame::MessageType & type);	//zostavenie a rozoslanie spravy o interakcii s okolim
	void SetupLight();
	void SetupMotionModel();			//inicializacia pouzitych motionmodelov
	void SetupPlayer();					//inicializacia hraca
	void SetupTreasure();				//inicializacia pokladu
	void SetupTrigger();				//inicializacia triggeru
	void SetupVehicles();				//inicializacia vozidiel
	void VehicleAction(bool enter);		//Interakcia s vozidlom

	void SetupCow();

	bool inGame;
	bool reset;
	bool houseEnter;
	bool wantChange;
	bool changeMap1, changeMap2;
	bool inv_key, inv_shovel;
	bool treasure_used;

	dtCore::RefPtr<dtCore::FPSMotionModel> fmm; //po svojich mm
	dtCore::RefPtr<dtCore::WalkMotionModel> wmm;//vozidlo mm
	dtCore::RefPtr<dtCore::SpotLight> light;	//baterka
	PlayerActor *player;
	RadarActor *radar;
	EnterTriggerActor *trigger;
	InteractiveActor *goal;
	InteractiveActor *intersectedObject; //pomocny ukazatel
	dtCore::UniqueId lastInteraction;
	dtCore::Transform VehiclePosition;
};

#endif