/*****************************************************************************\
 *
 * collectibleActor.cpp
 *
 * CollectibleActor - Actor class representing collectible object
 * CollectibleActorProxy - Proxy class
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include <iostream>
#include <dtCore/transform.h>
#include "collectibleActor.h"
#include "MyMessages.h"

CollectibleActor::CollectibleActor(dtGame::GameActorProxy &proxy)
	: InteractiveActor(proxy)
	, isCollected(false)
{
}

CollectibleActor::~CollectibleActor()
{
}

//spracovanie sprav
void CollectibleActor::ProcessMessage(const dtGame::Message &message)
{
	if(message.GetMessageType() == MyMessageType::ITEM_COLLECTED)
	{
		//ak je toto zodvihnuty predmet
		if(message.GetAboutActorId() == GetUniqueId())
		{
			//zmizne zo sceny
			SetCollected(true);
		}
	}
}

//nastavenie priznaku "zodvihnuty"
void CollectibleActor::SetCollected(bool set)
{
	SetTransform(dtCore::Transform(0.f, 0.f, -100.f, 0.f, 0.f, 0.f));
	if(set)
		ClearCollisionGeometry();
	SetActive(!set);
	isCollected = set;
}

/* ---------------------------------!!! COLLECTIBLE ACTOR PROXY !!!-------------------------------------- */

CollectibleActorProxy::CollectibleActorProxy()
{
}

CollectibleActorProxy::~CollectibleActorProxy()
{
}

void CollectibleActorProxy::BuildInvokables()
{
	InteractiveActorProxy::BuildInvokables();
}

void CollectibleActorProxy::BuildPropertyMap()
{
	InteractiveActorProxy::BuildPropertyMap();
}

void CollectibleActorProxy::CreateActor()
{
	SetActor(*new CollectibleActor(*this));
}

void CollectibleActorProxy::OnEnteredWorld()
{
	RegisterForMessages(MyMessageType::ITEM_COLLECTED);

	InteractiveActorProxy::OnEnteredWorld();
}