/*****************************************************************************\
 *
 * enterTriggerActor.cpp
 *
 * EnterTriggerActor - Actor class representing building entry/exit point
 * EnterTriggerActorProxy - Proxy class
 * EnterAction - Triggered action
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include <dtCore/scene.h>
#include <dtGame/gameapplication.h>
#include <dtGame/gamemanager.h>
#include "enterTriggerActor.h"
#include "MyMessages.h"

#include <iostream>

EnterAction::EnterAction()
{
}

EnterAction::~EnterAction()
{
}

void EnterAction::SetTrigger(dtGame::GameActor &TriggerActor)
{
	MyTrigger = &TriggerActor;
}

void EnterAction::OnStart()
{
	dtCore::RefPtr<dtGame::Message> msg = MyTrigger->GetGameActorProxy().GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::HOUSE_ENTER_POSSIBLE);
	MyTrigger->GetGameActorProxy().GetGameManager()->SendMessage(*msg);
}

//trigger

EnterTriggerActor::EnterTriggerActor(dtGame::GameActorProxy &proxy)
	: dtGame::GameActor(proxy)
	, computeDistance(false)
	, trigger(NULL)
	, action(NULL)
{
}

EnterTriggerActor::~EnterTriggerActor()
{
}

double EnterTriggerActor::CountDistance()
{
	double Xrange, Yrange, Zrange;
	dtCore::Transform xform;
	osg::Vec3 playerPos;

	//zistime aktualnu poziciu
	PlayerActor *pa = dynamic_cast<PlayerActor*>(GetGameActorProxy().GetGameManager()->FindActorById(player)->GetActor());
	if(pa)
	{
		pa->GetTransform(xform);
		xform.GetTranslation(playerPos);

	//spocitame vzdialenost od ciela
		Xrange = playerPos.x() - position.x();
		Yrange = playerPos.y() - position.y();
		Zrange = playerPos.z() - position.z();
		return sqrt((Xrange*Xrange)+(Yrange*Yrange)+(Zrange*Zrange));
	}
	else return -9999;
}

void EnterTriggerActor::OnTickLocal(const dtGame::TickMessage &tickMessage)
{
	double distance;
	if(computeDistance) // nejak sem dostat playerovu polohu
	{
		if((distance = CountDistance()) > 3)
		{
			dtCore::RefPtr<dtGame::Message> msg = GetGameActorProxy().GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::HOUSE_ENTER_IMPOSSIBLE);
			GetGameActorProxy().GetGameManager()->SendMessage(*msg);
			computeDistance = false;
		}//>3
	}//compute
}

void EnterTriggerActor::ProcessMessage(const dtGame::Message &message)
{
	if(message.GetMessageType() == MyMessageType::HOUSE_ENTER_POSSIBLE)
		computeDistance = true;
}

void EnterTriggerActor::Init(dtCore::UniqueId playerId)
{
	action = new EnterAction();
	action->SetTrigger(*this);
	trigger = new dtABC::ProximityTrigger("trigger");
	trigger->GetTrigger()->SetTimesActive(-1);
	trigger->SetCollisionSphere(2.f);

	dtCore::Transform xform;
	GetTransform(xform);
	xform.GetTranslation(position);
	player = playerId;
	trigger->GetTrigger()->SetAction(action.get());
	AddChild(trigger.get());
}

//proxy

EnterTriggerActorProxy::EnterTriggerActorProxy()
{
}

EnterTriggerActorProxy::~EnterTriggerActorProxy()
{
}

void EnterTriggerActorProxy::BuildInvokables()
{
	GameActorProxy::BuildInvokables();
}

void EnterTriggerActorProxy::BuildPropertyMap()
{
	GameActorProxy::BuildPropertyMap();
}

void EnterTriggerActorProxy::CreateActor()
{
	SetActor(*new EnterTriggerActor(*this));
}

void EnterTriggerActorProxy::OnEnteredWorld()
{
	RegisterForMessages(MyMessageType::HOUSE_ENTER_POSSIBLE);
	RegisterForMessages(dtGame::MessageType::TICK_LOCAL, dtGame::GameActorProxy::TICK_LOCAL_INVOKABLE);

	GameActorProxy::OnEnteredWorld();
}