/*****************************************************************************\
 *
 * enterTriggerActor.h
 *
 * EnterTriggerActor - Actor class representing building entry/exit point
 * EnterTriggerActorProxy - Proxy class
 * EnterAction - Triggered action
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#ifndef TRIGGER_H
#define TRIGGER_H

#include <dtABC/action.h>
#include <dtABC/proximitytrigger.h>
#include <dtGame/gameactor.h>

#include "export.h"
#include "playerActor.h"

//akcia, ktoru vykoname pri spusteni triggeru
class EnterAction : public dtABC::Action
{
public:
	EnterAction();

	void SetTrigger(dtGame::GameActor & TriggerActor);

protected:
	~EnterAction();

private:
	virtual void OnStart();
	virtual bool OnNextStep()  { return false; }
	virtual void OnPause()     {}
    virtual void OnUnPause()   {}

	dtCore::RefPtr<dtGame::GameActor> MyTrigger;
};

//trigger
class IBP_EXPORT EnterTriggerActor : public dtGame::GameActor
{
public:
	EnterTriggerActor(dtGame::GameActorProxy &proxy);
	~EnterTriggerActor();

	double CountDistance();
	virtual void OnTickLocal(const dtGame::TickMessage &tickMessage);
	virtual void ProcessMessage(const dtGame::Message &message);

	void Init(dtCore::UniqueId playerId);
private:
	bool computeDistance;

	dtCore::RefPtr<dtABC::ProximityTrigger> trigger;
	dtCore::RefPtr<EnterAction> action;
	dtCore::UniqueId player;
	osg::Vec3 position;
};

//proxy
class IBP_EXPORT EnterTriggerActorProxy : public dtGame::GameActorProxy
{
public:
	EnterTriggerActorProxy();
	~EnterTriggerActorProxy();

	virtual void BuildInvokables();
	virtual void BuildPropertyMap();
	virtual void CreateActor();
	virtual void OnEnteredWorld();
};

#endif