/*****************************************************************************\
 *
 * playerActor.cpp
 *
 * PlayerActor - Class representing player entity
 * PlayerActorProxy - Proxy class
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include "MyMessages.h"
#include "playerActor.h"
#include "interactiveActor.h"
#include <dtCore/refptr.h>
#include <dtCore/scene.h>
#include <dtGame/basemessages.h>
#include <dtGame/gameapplication.h>
#include <iostream>

/* ---------------------------------!!! PLAYER ACTOR !!!-------------------------------------- */

PlayerActor::PlayerActor(dtGame::GameActorProxy &proxy)
	: dtGame::GameActor(proxy)
	, collision(false)
	, key(false)
	, shovel(false)
	, IntersectRay(new dtCore::Isector)
	, inVehicle(false)
{
}

PlayerActor::~PlayerActor()
{
}

//zachytavanie nizkourovnovych sprav
void PlayerActor::OnMessage(dtCore::Base::MessageData *data)
{
	if(data->message == "collision")	//kolizia
	{
		dtCore::Scene::CollisionData* cd = static_cast<dtCore::Scene::CollisionData*>(data->userData);
		
		if((cd->mBodies[0]->GetName() == GetGameActorProxy().GetName()) || (cd->mBodies[1]->GetName() == GetGameActorProxy().GetName())) //kolidoval tento objekt
		{
			SetCollision(true);
		}//name
	} //collision
}

//tick
void PlayerActor::OnTickLocal(const dtGame::TickMessage &tickMessage)
{
	Update(); //riesime kolizie/posunutie
	if(!inVehicle)
	{
		Intersect();	//interakcia s okolim
	}
}

//spracovanie sprav
void PlayerActor::ProcessMessage(const dtGame::Message &message)
{
	if(message.GetMessageType() == MyMessageType::ITEM_COLLECTED)
	{
		//ak je kluc
		if(GetGameActorProxy().GetGameManager()->FindActorById(message.GetAboutActorId())->GetActor()->GetName() == "Key")
		{
			SetHasKey(true);
		}
		//ak je lopata
		if(GetGameActorProxy().GetGameManager()->FindActorById(message.GetAboutActorId())->GetActor()->GetName() == "Shovel")
		{
			SetHasShovel(true);
		}
	}
	else if(message.GetMessageType() == MyMessageType::VEHICLE_ENTERED)
	{
		SetIsInVehicle(true);
	}
	else if(message.GetMessageType() == MyMessageType::VEHICLE_LEFT)
	{
		SetIsInVehicle(false);
	}
	else if(message.GetMessageType() == dtGame::MessageType::INFO_TIMER_ELAPSED)
	{
		dtCore::RefPtr<dtGame::Message> msg = GetGameActorProxy().GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::HIDE_HINT);
		msg->SetAboutActorId(GetUniqueId());
		GetGameActorProxy().GetGameManager()->SendMessage(*msg);
	}
}

//inicializacia
void PlayerActor::Init()
{
	//inicializacia
	GetTransform(oldPosition);
	GetTransform(newPosition);
	
	if(IntersectRay == NULL)
		IntersectRay = new dtCore::Isector;
	IntersectRay->SetScene(&GetGameActorProxy().GetGameManager()->GetScene());
	IntersectRay->SetLength(3.f);
	
	//zavesime kameru
	AddChild(GetGameActorProxy().GetGameManager()->GetApplication().GetCamera());
	//registrujeme sa pre odchytavanie sprav o koliziach
	AddSender(&GetGameActorProxy().GetGameManager()->GetScene());
}

//picking
void PlayerActor::Intersect()
{
	if(IntersectRay){
		IntersectRay->Reset();	//nutne zavolat pred kazdym "pouzitim" Isectoru

		osg::Vec3 trans, rot;
		osg::Matrix rotMatrix;
		newPosition.GetTranslation(trans);
		newPosition.GetRotation(rotMatrix);

		rot.set(rotMatrix(1,0), rotMatrix(1,1), rotMatrix(1,2)); //smer

		IntersectRay->SetStartPosition(trans);
		IntersectRay->SetDirection(rot);

		if(IntersectRay->Update())
		{
			dtCore::DeltaDrawable *item = IntersectRay->GetClosestDeltaDrawable();
			//testovat na interaktivny objekt
			if(item != NULL)
			{
				InteractiveActor *ia = dynamic_cast<InteractiveActor*>(item);
				if(ia)
				{
					dtCore::RefPtr<dtGame::Message> msg = GetGameActorProxy().GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::INTERSECTED);
					msg->SetAboutActorId(ia->GetUniqueId());	//vlozime do spravy id konkretneho objektu
					GetGameActorProxy().GetGameManager()->SendMessage(*msg);
				}
			}
		}else{
			dtCore::RefPtr<dtGame::Message> msg = GetGameActorProxy().GetGameManager()->GetMessageFactory().CreateMessage(MyMessageType::NOT_INTERSECTED);
			GetGameActorProxy().GetGameManager()->SendMessage(*msg);
		}//update
	}//ray
}

//nastavenie collision flag
void PlayerActor::SetCollision(bool col)
{
	collision = col;
}

//riesenie kolizii
void PlayerActor::Update()
{
	osg::Vec3 playerPosition;
	if(collision && !inVehicle)
	{
		SetTransform(oldPosition);	//napozicovanie
		SetCollision(false);		//zvesime flag
	}else{
		oldPosition = newPosition;	//posunutie
		GetTransform(newPosition);
		newPosition.GetTranslation(playerPosition);
	}
}

//ma v inventari kluc?
bool PlayerActor::GetHasKey()
{
	return key;
}

//ma v inventari lopatu?
bool PlayerActor::GetHasShovel()
{
	return shovel;
}

//je vo vozidle?
bool PlayerActor::IsInVehicle()
{
	return inVehicle;
}

//pridanie kluca do inventaru
void PlayerActor::SetHasKey(bool set)
{
	key = set;
}

//pridanie lopaty do inventaru
void PlayerActor::SetHasShovel(bool set)
{
	shovel = set;
}

//nastavenie priznaku o pouziti vozidla
void PlayerActor::SetIsInVehicle(bool set)
{
	inVehicle = set;
}

/* ---------------------------------!!! PLAYER ACTOR PROXY !!!-------------------------------------- */

PlayerActorProxy::PlayerActorProxy()
{
}

PlayerActorProxy::~PlayerActorProxy()
{
}

void PlayerActorProxy::BuildPropertyMap()
{
	dtGame::GameActorProxy::BuildPropertyMap();
}

void PlayerActorProxy::BuildInvokables()
{
	dtGame::GameActorProxy::BuildInvokables();
}

void PlayerActorProxy::CreateActor()
{
	SetActor(*new PlayerActor(*this));
}

void PlayerActorProxy::OnEnteredWorld()
{
	RegisterForMessages(MyMessageType::ITEM_COLLECTED);
	RegisterForMessages(MyMessageType::VEHICLE_ENTERED);
	RegisterForMessages(MyMessageType::VEHICLE_LEFT);
	RegisterForMessages(dtGame::MessageType::INFO_TIMER_ELAPSED);
	RegisterForMessages(dtGame::MessageType::TICK_LOCAL, dtGame::GameActorProxy::TICK_LOCAL_INVOKABLE);

	dtGame::GameActorProxy::OnEnteredWorld();
}