/*****************************************************************************\
 *
 * playerActor.h
 *
 * PlayerActor - Class representing player entity
 * PlayerActorProxy - Proxy class
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#ifndef PLAYER_ACTOR_H
#define PLAYER_ACTOR_H

#include <dtCore/isector.h>
#include <dtCore/transform.h>
#include <dtGame/gameactor.h>
#include "export.h"
#include "collectibleActor.h"

class IBP_EXPORT PlayerActor : public dtGame::GameActor
{
public:
	PlayerActor(dtGame::GameActorProxy &proxy);

	virtual void OnMessage(dtCore::Base::MessageData *data);			//"nizkourovnove" spravy
	virtual void OnTickLocal(const dtGame::TickMessage &tickMessage);	//tick
	virtual void ProcessMessage(const dtGame::Message &message);		//spracovanie sprav

	void Init();					//inicializacia hraca
	void Intersect();				//"picking" objektov z okolia
	void SetCollision(bool col);	//nastavenie collision flagu
	void Update();					//riesnie kolizii

	bool GetHasKey();
	bool GetHasShovel();
	void SetHasKey(bool set);
	void SetHasShovel(bool set);
	bool IsInVehicle();
	void SetIsInVehicle(bool set);

protected:
	~PlayerActor();

private:
	bool collision;
	bool key, shovel;
	bool inVehicle;
	dtCore::Transform oldPosition, newPosition;
	dtCore::RefPtr<dtCore::Isector> IntersectRay;
};

/*-----------------------!!! PLAYER ACTOR PROXY !!!--------------------------*/

class IBP_EXPORT PlayerActorProxy : public dtGame::GameActorProxy
{
public:
	PlayerActorProxy();
	~PlayerActorProxy();

	virtual void BuildPropertyMap();
	virtual void BuildInvokables();
	virtual void CreateActor();

	virtual void OnEnteredWorld();
};

#endif