/*****************************************************************************\
 *
 * radarActor.cpp
 *
 * RadarActor - Class representing treasure detector
 * RadarActorProxy - Proxy class
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include <dtAudio/audiomanager.h>
#include <dtCore/transform.h>
#include <dtGame/messagetype.h>
#include <iostream>
#include <math.h>
#include "MyMessages.h"
#include "radarActor.h"

RadarActor::RadarActor(dtGame::GameActorProxy &proxy)
	: dtGame::GameActor(proxy)
	, enabled(false)
	, radarPosition(0.f, 0.f, 0.f)
	, goalPosition(0.f, 0.f, 0.f)
	, beepSound(NULL)
{
	counter = 0;
}

RadarActor::~RadarActor()
{
}

//spracovanie sprav
void RadarActor::ProcessMessage(const dtGame::Message &message)
{
	if(message.GetMessageType() == MyMessageType::TOGGLE_RADAR)
	{
		Enable(!IsEnabled());
	}
}

void RadarActor::OnTickLocal(const dtGame::TickMessage &tickMessage)
{
	if(enabled)
	{
		CountDistance();		//spocitame vzdialenost
		//pripocitame zodpovedajucu hodnotu k citacu
		if(distance > 50)
			counter += 2;
		else if(distance > 30)
			counter += 4;
		else if(distance > 20)
			counter += 6;
		else if(distance > 10)
			counter += 8;
		else counter += 15;
		if(counter >= 200)				//ak dosiahol kriticku hranicu
		{
			PlayBeep();					//pipneme
			counter = 0;
		}
	}
}

//spocitame vzdialenost od pokladu
void RadarActor::CountDistance()
{
	double Xrange, Yrange, Zrange;
	dtCore::Transform xform;

	//zistime aktualnu poziciu
	GetTransform(xform);
	xform.GetTranslation(radarPosition);

	//spocitame vzdialenost od ciela
	Xrange = radarPosition.x() - goalPosition.x();
	Yrange = radarPosition.y() - goalPosition.y();
	Zrange = radarPosition.z() - goalPosition.z();
	distance = sqrt((Xrange*Xrange)+(Yrange*Yrange)+(Zrange*Zrange));
}

//zapnutie/vypnutie radaru
void RadarActor::Enable(bool enable)
{
	enabled = enable;
	if(enable)
		std::cout<<"Radar enabled"<<std::endl;
	else std::cout<<"Radar disabled"<<std::endl;
}

//inicializacia
void RadarActor::Init()
{
	beepSound = dtAudio::AudioManager::GetInstance().NewSound();
	beepSound->LoadFile("Sounds/carhorn.wav");
	beepSound->SetLooping(false);
	AddChild(beepSound.get());
}

//vrati info o stave
bool RadarActor::IsEnabled()
{
	return enabled;
}

//pipnutie a vynulovanie citaca
void RadarActor::PlayBeep()
{
	//prehraj zvuk
	if(beepSound)
	{
		beepSound->Play();
	}
}

//nastavenie pozicie ciela
void RadarActor::SetGoal(osg::Vec3 &goal)
{
	goalPosition = goal;
}

/*-----------------------!!! RADAR ACTOR PROXY !!!--------------------------*/

RadarActorProxy::RadarActorProxy()
{
}

RadarActorProxy::~RadarActorProxy()
{
}

void RadarActorProxy::BuildPropertyMap()
{
	GameActorProxy::BuildPropertyMap();
}

void RadarActorProxy::BuildInvokables()
{
	GameActorProxy::BuildInvokables();
}

void RadarActorProxy::CreateActor()
{
	SetActor(*new RadarActor(*this));
}

void RadarActorProxy::OnEnteredWorld()
{
	RegisterForMessages(MyMessageType::TOGGLE_RADAR);
	RegisterForMessages(dtGame::MessageType::TICK_LOCAL, dtGame::GameActorProxy::TICK_LOCAL_INVOKABLE);

	GameActorProxy::OnEnteredWorld();
}
