/*****************************************************************************\
 *
 * radarActor.h
 *
 * RadarActor - Class representing treasure detector
 * RadarActorProxy - Proxy class
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#ifndef RADAR_ACTOR_H
#define RADAR_ACTOR_H

#include <dtAudio/sound.h>
#include <dtCore/transform.h>
#include <dtGame/gameactor.h>
#include <dtGame/message.h>
#include "export.h"

class IBP_EXPORT RadarActor : public dtGame::GameActor
{
public:
	RadarActor(dtGame::GameActorProxy &proxy);

	virtual void ProcessMessage(const dtGame::Message &message);
	virtual void OnTickLocal(const dtGame::TickMessage &tickMessage);

	void Enable(bool enable);		//zapne/vypne radar
	void Init();					//inicializacia
	bool IsEnabled();				//vrati info o stave
	void SetGoal(osg::Vec3 & goal);		//nastavi suradnice ciela

protected:
	~RadarActor();
private:
	void CountDistance();			//spocita vzdialenost od ciela
	void PlayBeep();				//pipnutie + vynulovanie citaca

	bool enabled;
	int counter;
	double distance;
	osg::Vec3 radarPosition;
	osg::Vec3 goalPosition;
	dtCore::RefPtr<dtAudio::Sound> beepSound;
};

/*-----------------------!!! RADAR ACTOR PROXY !!!--------------------------*/

class IBP_EXPORT RadarActorProxy : public dtGame::GameActorProxy
{
public:
	RadarActorProxy();
	~RadarActorProxy();

	virtual void BuildPropertyMap();
	virtual void BuildInvokables();
	virtual void CreateActor();

	virtual void OnEnteredWorld();
};

#endif