/*****************************************************************************\
 *
 * treasureActor.cpp
 *
 * TreasureActor - Class representing treasure
 * TreasureActorProxy - Proxy class
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include "treasureActor.h"
#include "MyMessages.h"
#include <dtAudio/audiomanager.h>
#include <dtCore/transform.h>
#include <iostream>

TreasureActor::TreasureActor(dtGame::GameActorProxy &proxy)
	: InteractiveActor(proxy)
	, used(false)
	, explosionSound(NULL)
	, explosionSystem(NULL)
{
}

TreasureActor::~TreasureActor()
{
}

//inicializacia
void TreasureActor::Init()
{
	explosionSound = dtAudio::AudioManager::GetInstance().NewSound();
	explosionSound->LoadFile("Sounds/exp57.wav");
	explosionSound->SetLooping(false);
	AddChild(explosionSound.get());

	explosionSystem = new dtCore::ParticleSystem("explosion");
	explosionSystem->LoadFile("Particles/vybuch.osg");
	explosionSystem->SetEnabled(false);
	explosionSystem->ResetTime();
	AddChild(explosionSystem.get());
}

bool TreasureActor::IsUsed()
{
	return used;
}

//Aktivacia/pouzitie objektu
void TreasureActor::Use(bool reset)
{
	//"vykopanie" pokladu
	osg::Vec3 transl;
	dtCore::Transform xform;
	GetTransform(xform);
	xform.GetTranslation(transl);
	if(!reset)
		transl.z() = transl.z() + 0.5f;
	else
		transl.z() = transl.z() - 0.5f;
	xform.SetTranslation(transl);
	SetTransform(xform);

	SetUsed(true);
}

//zaznamenanie pouzitia
void TreasureActor::SetUsed(bool use)
{
	used = use;
}

//vybuch
void TreasureActor::Explode()
{
	if(explosionSound && explosionSystem)
	{
		explosionSystem->SetEnabled(true);
		explosionSound->Play();
	}
}

void TreasureActor::ProcessMessage(const dtGame::Message &message)
{
}

/*------------------------ !!! USABLE ACTOR PROXY !!! -----------------------*/

TreasureActorProxy::TreasureActorProxy()
{
}

TreasureActorProxy::~TreasureActorProxy()
{
}

void TreasureActorProxy::BuildInvokables()
{
	InteractiveActorProxy::BuildInvokables();
}

void TreasureActorProxy::BuildPropertyMap()
{
	InteractiveActorProxy::BuildPropertyMap();
}

void TreasureActorProxy::CreateActor()
{
	SetActor(*new TreasureActor(*this));
}

void TreasureActorProxy::OnEnteredWorld()
{
	InteractiveActorProxy::OnEnteredWorld();
}