/*****************************************************************************\
 *
 * vehicleActor.h
 *
 * VehicleActor - Class representing usable vehicle
 * VehicleActorProxy - Proxy class
 *
 * Authors: Luk Martanovi (xmarta00 _AT stud.fit.vutbr.cz)
 *
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#ifndef VEHICLE_ACTOR_H
#define VEHICLE_ACTOR_H

#include <dtAudio/sound.h>
#include <dtCore/particlesystem.h>
#include <dtCore/transform.h>
#include <dtGame/message.h>
#include "interactiveActor.h"
#include "export.h"

class IBP_EXPORT VehicleActor : public InteractiveActor
{
public:
	VehicleActor(dtGame::GameActorProxy &proxy);
	
	virtual void OnMessage(dtCore::Base::MessageData *data);			//"nizkourovnove" spravy
	virtual void OnTickLocal(const dtGame::TickMessage &tickMessage);	//tick
	virtual void ProcessMessage(const dtGame::Message &message);		//spracovanie hernych sprav

	void Init();					//inicializacia pozicie a sceny
	bool GetIsActive();				//je aktivny?
	void SetIsActive(bool set);		//prepinac aktivny/neaktivny
	void SetCollision(bool set);	//nastavenie priznaku o koliziach
	void Update();					//riesenie kolizii
	
	void SetSoundFile();
	void SetIsLand(bool set);		//nastavi, ze vozidlo je pozemne
	bool GetIsLand();				//je pozemne?

protected:
	~VehicleActor();

private:
	bool isActive;
	bool collision;
	bool isLand;
	bool isMoving;

	dtCore::Transform oldPosition, newPosition, initPosition;
	dtCore::RefPtr<dtCore::ParticleSystem> dust;
	dtCore::RefPtr<dtAudio::Sound> engineSound;
};

/*------------------------ !!! VEHICLE ACTOR PROXY !!! -----------------------*/

class IBP_EXPORT VehicleActorProxy : public InteractiveActorProxy
{
public:
	VehicleActorProxy();
	~VehicleActorProxy();

	virtual void BuildInvokables();
	virtual void BuildPropertyMap();
	virtual void CreateActor();

	virtual void OnEnteredWorld();
};

#endif